import random
from combat import spell_classes


# ******** Combat Mechanics ********
class CombatEncounter(object):
    def __init__(self, PC, mob):
        self.player = PC
        self.mob = mob
        self.player_defended_recently = False
        self.player_defence_cost = 1
        self.mob_defended_recently = False
        self.turn_counter = 0
        self.spell_invoker = spell_classes.SpellInvoker(self.player, self.mob)
        self.temp_effect_string = ""
        self.temp_effect_damage = 0
        self.temp_effect_duration = 0

    def calc_damage_to_mob(self):
        damage = self.player.attack + random.choice(range(1, self.player.dexterity))
        return int(damage)

    def calc_damage_to_PC(self):
        damage = self.mob.attack + random.choice(range(1, ((self.mob.level * 2) + 1)))
        return int(damage)

    def calc_mob_HP_taken(self, damage):
        hp_taken = damage - damage * (self.mob.defence / 150)

        if hp_taken > 0:
            return int(hp_taken)
        else:
            return 1

    def calc_PC_HP_taken(self, damage):
        hp_taken = damage - damage * (self.player.defence / 150)
        if hp_taken > 0:
            return int(hp_taken)
        else:
            return 1

    def action_menu(self):
        # Player selects option
        print("""
   Select an option:
   1: Attack (-2 turn)
   2: Defend (+1 turn)
   3: Spells
   4: Items
   5: Run
   6: Do nothing?
           \n""")
        player_select = input()

        # Attack
        if player_select == "1":
            damage_dealt = self.calc_mob_HP_taken(self.calc_damage_to_mob())
            print(self.player.player_name + " attacks, dealing " + str(damage_dealt) + " damage.")
            self.mob.current_health -= damage_dealt
            if self.player.turn >= -8:
                self.player.turn -= 2
            self.player_defence_cost = 1
            self.player_defended_recently = False
            input()

        # Defend
        elif player_select == "2":
            if self.player_defended_recently:
                print(self.player.player_name + " braces for the attack again, but it has no effect\n")
            else:
                print(self.player.player_name + " braces for an attack.\n")
            self.player.defence += self.player_defence_cost
            if self.player.turn <= 9:
                self.player.turn += self.player_defence_cost
            self.player_defence_cost = 0
            self.player_defended_recently = True
            input()

        # Spells
        elif player_select == "3":
            self.spell_invoker.menu()

            self.temp_effect_string = self.spell_invoker.temp_effect_string
            self.temp_effect_damage = self.spell_invoker.temp_effect_damage
            self.temp_effect_duration += self.spell_invoker.temp_effect_duration
            input()

        # Items
        elif player_select == "4":
            i = 1
            print("\n\nWhich would you like to use?")
            for item in self.player.inventory:
                print("    " + str(i) + ": " + item.name + " x " + str(item.quantity))
                i += 1
            while True:
                try:
                    player_select = input()
                    run_choice = self.player.inventory[int(player_select) - 1]
                    player_num = input("And how many?\n\n\n")
                    run_choice.use(int(player_num))
                except IndexError:
                    print("Try again")
                except ValueError:
                    print("Try again")
                else:
                    break

        # Do nothing
        elif player_select == "6":
            print(self.player.player_name + " stands completely still, trying to blend in with the surroundings.\n")
            if self.player.turn >= -9:
                self.player.turn -= 1
            self.player_defence_cost = 1
            self.player_defended_recently = False
            input()

        else:
            self.action_menu()

    def combat_stats(self):
        print(self.player.player_name + ":")
        print("________________________________________")
        print("Turn:    " + str(self.player.turn))
        print("HP:      " + str(self.player.current_health) + "/" + str(self.player.max_health))
        print("AP:      " + str(self.player.ability_power) + "/" + str(self.player.max_ability_power))
        print("Attack:  " + str(self.player.attack))
        print("Defence: " + str(self.player.defence))
        print("________________________________________\n\n")

        print(self.mob.name + ":")
        print("________________________________________")
        print("Turn:    " + str(self.mob.turn))
        print("HP:      " + str(self.mob.current_health) + "/" + str(self.mob.max_health))
        print("________________________________________\n\n")
        input()

    def combat(self):
        player_first = True
        if self.player.turn < self.mob.turn:
            player_first = False

        # Player's Turn
        if player_first:
            print("It's " + self.player.player_name + "'s turn")
            input()
            self.combat_stats()
            input()

            # The main menu
            self.action_menu()

        # Mob's Turn
        else:
            # Status effects
            if self.temp_effect_duration > 0:
                print("\nTemp status effects:\n_______________")
                print(self.temp_effect_string)
                self.mob.current_health -= self.temp_effect_damage
                self.temp_effect_duration -= 1

            print("It's " + self.mob.name + "'s turn.")
            input()
            mob_choices = [1, 2]
            if self.mob_defended_recently:
                mob_choices.remove(2)
            else:
                mob_choices.append(2)

            # Random mob ability or action is selected
            mob_select = random.choice(mob_choices)  # start int inclusive, end int exclusive(so make it +1 of desired)

            # Mob inflicts damage to player
            if mob_select == 1:
                damage_dealt = self.calc_PC_HP_taken(self.calc_damage_to_PC())
                print(self.mob.name + " attacks, dealing " + str(damage_dealt) + " damage.")
                self.player.current_health -= damage_dealt
                if self.mob.turn >= -8:
                    self.mob.turn -= 2
                self.mob_defended_recently = False

            if mob_select == 2:
                print(self.mob.name + " braces for an attack.\n")
                self.mob.defence += 1
                if self.mob.turn <= 9:
                    self.mob.turn += 1
                self.mob_defended_recently = True
            input()
            self.combat_stats()
            input()


# Even easier method for combat
def quick_combat(mob, player):
    combat = CombatEncounter(player, mob)

    while (player.current_health > 0) and (mob.current_health > 0):
        combat.combat()
        if player.current_health <= 0:
            print("The " + mob.name + " has killed you.\nYou died a terrible and painful death")
            return False
        if mob.current_health <= 0:
            print("The " + mob.name + " slumps over, dead.")
            print("You've gained " + str(mob.XP_given) + "XP")
            player.leveler(mob.XP_given)
            return True
